import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { Inject } from '@nestjs/common';
import { CommandHandler } from '@nestjs/cqrs';
import { AdsRepository } from '../../adapters/secondaries/ads.repository';
import { CreateAdCommand } from '../../commands/create-ad.command';
import { CreateAdRequest } from '../dtos/create-ad.request';
import { IProvideParams } from '../interfaces/param-provider.interface';
import { DefaultParams } from '../types/default-params.type';
import { AdCreation } from '../dtos/ad.creation';
import { Ad } from '../entities/ad';
import { PARAMS_PROVIDER } from '../../ad.constants';
import { IPublishMessage } from '../../../../interfaces/message-publisher';
import { MESSAGE_PUBLISHER } from '../../../../app.constants';

@CommandHandler(CreateAdCommand)
export class CreateAdUseCase {
  private readonly defaultParams: DefaultParams;
  private ad: AdCreation;
  constructor(
    private readonly repository: AdsRepository,
    @Inject(MESSAGE_PUBLISHER)
    private readonly messagePublisher: IPublishMessage,
    @InjectMapper() private readonly mapper: Mapper,
    @Inject(PARAMS_PROVIDER)
    private readonly defaultParamsProvider: IProvideParams,
  ) {
    this.defaultParams = defaultParamsProvider.getParams();
  }

  async execute(command: CreateAdCommand): Promise<Ad> {
    this.ad = this.mapper.map(
      command.createAdRequest,
      CreateAdRequest,
      AdCreation,
    );
    this.setDefaultMarginDurations();
    this.setDefaultAddressesPosition();
    this.setDefaultDriverAndPassengerParameters();
    this.setDefaultStrict();

    try {
      const adCreated: Ad = await this.repository.create(this.ad);
      this.messagePublisher.publish('ad.create', JSON.stringify(adCreated));
      this.messagePublisher.publish(
        'logging.ad.create.info',
        JSON.stringify(adCreated),
      );
      return adCreated;
    } catch (error) {
      let key = 'logging.ad.create.crit';
      if (error.message.includes('Already exists')) {
        key = 'logging.ad.create.warning';
      }
      this.messagePublisher.publish(
        key,
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }

  private setDefaultMarginDurations = (): void => {
    if (this.ad.monMargin === undefined)
      this.ad.monMargin = this.defaultParams.MON_MARGIN;
    if (this.ad.tueMargin === undefined)
      this.ad.tueMargin = this.defaultParams.TUE_MARGIN;
    if (this.ad.wedMargin === undefined)
      this.ad.wedMargin = this.defaultParams.WED_MARGIN;
    if (this.ad.thuMargin === undefined)
      this.ad.thuMargin = this.defaultParams.THU_MARGIN;
    if (this.ad.friMargin === undefined)
      this.ad.friMargin = this.defaultParams.FRI_MARGIN;
    if (this.ad.satMargin === undefined)
      this.ad.satMargin = this.defaultParams.SAT_MARGIN;
    if (this.ad.sunMargin === undefined)
      this.ad.sunMargin = this.defaultParams.SUN_MARGIN;
  };

  private setDefaultStrict = (): void => {
    if (this.ad.strict === undefined)
      this.ad.strict = this.defaultParams.STRICT;
  };

  private setDefaultDriverAndPassengerParameters = (): void => {
    this.ad.driver = !!this.ad.driver;
    this.ad.passenger = !!this.ad.passenger;
    if (!this.ad.driver && !this.ad.passenger) {
      this.ad.driver = this.defaultParams.DRIVER;
      this.ad.seatsDriver = this.defaultParams.SEATS_PROVIDED;
      this.ad.passenger = this.defaultParams.PASSENGER;
      this.ad.seatsPassenger = this.defaultParams.SEATS_REQUESTED;
      return;
    }
    if (!this.ad.seatsDriver || this.ad.seatsDriver <= 0)
      this.ad.seatsDriver = this.defaultParams.SEATS_PROVIDED;
    if (!this.ad.seatsPassenger || this.ad.seatsPassenger <= 0)
      this.ad.seatsPassenger = this.defaultParams.SEATS_REQUESTED;
  };

  private setDefaultAddressesPosition = (): void => {
    if (this.ad.addresses.create[0].position === undefined) {
      for (let i = 0; i < this.ad.addresses.create.length; i++) {
        this.ad.addresses.create[i].position = i;
      }
    }
  };
}
